import unittest


from Stackml.Stackml.Node import Node, EnergylayerElement



class TestNode(unittest.TestCase):

    def setUp(self):
        self.node = Node('1', 1, 'ThingNode')

    def test_add_layer_element_should_add_the_specific_element_type(self):
        # Assign
        element_type = 'EnergylayerElement'
        element = 'Buz'

        # Act
        e = self.node.add_layer_element(element, element_type, hub='False')

        # Assert
        self.assertIsInstance(e, EnergylayerElement)

    def test_get_layer_element_should_return_the_specific_element_type(self):
        # Assign
        element_type = 'EnergylayerElement'
        element = 'Buz'
        self.node.add_layer_element(element, element_type, hub='EndPoint')

        # Act
        e = self.node.get_layer_element(element_type)

        # Assert
        self.assertIsInstance(e, EnergylayerElement)



#Copyright STACKEO - INRIA 2020 .
from Stackml.Stackml.LayerElement import LayerElement
from Stackml.Stackml.LayerElement.ConnectivitylayerElement import ConnectivitylayerElement
from Stackml.Stackml.LayerElement.DatalayerElement import DatalayerElement
from Stackml.Stackml.LayerElement.EnergylayerElement import EnergylayerElement
from Stackml.Stackml.LayerElement.MessaginglayerElement import MessaginglayerElement
from Stackml.Stackml.LayerElement.NetworklayerElement import NetworklayerElement
from Stackml.Stackml.LayerElement.PhysicallayerElement import PhysicallayerElement
from Stackml.Stackml.LayerElement.ServicelayerElement import ServicelayerElement
from Stackml.Stackml.LayerElement.BusinesslayerElement import BusinesslayerElement


class Node:

    def __init__(self, id_: str, cardinality: int, type_: str):
        self.id_ = id_
        self.cardinality = cardinality
        self.element_types = {'BusinesslayerElement': BusinesslayerElement,
                              'ServicelayerElement': ServicelayerElement,
                              'DatalayerElement': DatalayerElement,
                              'MessaginglayerElement': MessaginglayerElement,
                              'ConnectivitylayerElement': ConnectivitylayerElement,
                              'NetworklayerElement': NetworklayerElement,
                              'PhysicallayerElement': PhysicallayerElement,
                              'EnergylayerElement': EnergylayerElement}
        self.layers_element = []
        self.type = type_
        self.nature = ''

    def add_layer_element(self, element, type_, hub) -> LayerElement:
        layer_element = self.element_types[type_](element, hub)
        self.layers_element.append(layer_element)
        return layer_element

    def get_layer_element(self, type_: str):
        layer = None
        for layer_ in self.layers_element:
            if layer_.type == type_:
                layer = layer_
                break
        return layer

    def get_components(self, layer_type: str):
        components = None
        layer_elem = self.get_layer_element(layer_type)
        if layer_elem:
            components = layer_elem.components
        return components

    def set_nature(self, nature: str) -> None:
        self.nature = nature



import os
import shutil
import tempfile

from decorator import contextmanager

SCRIPT_DIRECTORY_PATH = os.path.dirname(os.path.realpath(__file__))

@contextmanager
def clone_fixture(fixture_name, working_dir=None):
    tmp_prefix = '{0}_{1}'.format(fixture_name, '_')
    working_directory = tempfile.mktemp(prefix=tmp_prefix) if not working_dir else working_dir
    template_working_directory = os.path.join(SCRIPT_DIRECTORY_PATH, fixture_name)
    if not os.path.isdir(template_working_directory):
        fixtures_list = [d for d in os.listdir(SCRIPT_DIRECTORY_PATH) if
                         os.path.isdir(os.path.join(SCRIPT_DIRECTORY_PATH, d))]
        raise Exception('the fixture {0} does not exists in Fixtures : {1}'.format(fixture_name, fixtures_list))
    shutil.copytree(template_working_directory, working_directory)
    yield working_directory
    shutil.rmtree(working_directory)



from unittest import TestCase

from Stackml.Drawio.Component import Component
from Stackml.Drawio.Layer import Layer


class TestLayer(TestCase):

    def setUp(self):
        self.layer = Layer(type_="Physical", hub=False)

    def test_set_component_position_should_set_the_components_beside_when_layer_is_not_hub(self):
        # Assign
        c = Component('cmp1', 'type')
        x0 = c.position[0]
        y0 = c.position[1]

        #Act
        self.layer.set_component_position(c)

        #Assert
        self.assertEqual(x0, c.position[0])
        self.assertEqual(c.position[1], y0 + c.dimension[1] * len(self.layer.components) + (len(self.layer.components) +1 ) * 10)



    def test_set_component_position_should_set_the_components_in_stack_when_layer_is_hub(self):
        # Assign
        self.layer.hub = True
        c = Component('cmp1', 'type')
        y0 = c.position[1]
        x0 = c.position[0]
        d0 = c.dimension[0]

        #Act
        self.layer.set_component_position(c)

        #Assert
        self.assertEqual(y0, c.position[1])
        self.assertEqual(c.position[0], x0 + (d0 + 10) * len(self.layer.components) - 8)

    def test_add_component_position_should_increase_layer_height_when_they_are_more_then_one_component(self):
        # Assign
        height = self.layer.dimension[1]

        #Act
        self.layer.add_component('cmp1', 'type')
        c2 = self.layer.add_component('cmp2', 'type')

        #Assert
        self.assertEqual(self.layer.dimension[1], 10 + height + c2.dimension[1])
